import { defineStore } from "pinia";
import type {
  CreateOrUpdateCommunicationTypeViewModel,
  CommunicationTypeViewModel,
} from "../../viewmodels/admin/communicationType.models";
import { http } from "../../serverCom";

export const useCommunicationTypeStore = defineStore("communicationType", {
  state: () => {
    return {
      communicationTypes: [] as Array<CommunicationTypeViewModel>,
      communicationType: null as null | CommunicationTypeViewModel,
      availableFields: [] as Array<string>,
      loadingAll: "loading" as "loading" | "fetched" | "failed",
      loadingSingle: "loading" as "loading" | "fetched" | "failed",
      loadingFields: "loading" as "loading" | "fetched" | "failed",
      createStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
      updateStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
      deleteStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
    };
  },
  actions: {
    resetStatus() {
      this.createStatus = null;
      this.updateStatus = null;
      this.deleteStatus = null;
    },
    fetchCommunicationTypes() {
      this.loadingAll = "loading";
      http
        .get("/admin/communicationType")
        .then((result) => {
          this.communicationTypes = result.data;
          this.loadingAll = "fetched";
        })
        .catch((err) => {
          this.loadingAll = "failed";
        });
    },
    fetchCommunicationTypeById(id: number) {
      this.communicationType = null;
      this.loadingSingle = "loading";
      http
        .get(`/admin/communicationType/${id}`)
        .then((result) => {
          this.communicationType = result.data;
          this.loadingSingle = "fetched";
        })
        .catch((err) => {
          this.loadingSingle = "failed";
        });
    },
    fetchAvailableFields() {
      this.loadingFields = "loading";
      http
        .get("/admin/communicationType/fields")
        .then((result) => {
          this.availableFields = result.data;
          this.loadingFields = "fetched";
        })
        .catch((err) => {
          this.loadingFields = "failed";
        });
    },
    createCommunicationType(communicationType: CreateOrUpdateCommunicationTypeViewModel) {
      this.createStatus = "loading";
      http
        .post(`/admin/communicationType`, {
          communicationType: communicationType.type,
          fields: communicationType.fields,
        })
        .then((result) => {
          this.createStatus = { status: "success" };
          this.fetchCommunicationTypes();
        })
        .catch((err) => {
          this.createStatus = { status: "failed" };
        });
    },
    updateActiveCommunicationType(communicationType: CreateOrUpdateCommunicationTypeViewModel) {
      if (this.communicationType == null) return;
      let id = this.communicationType.id;
      this.updateStatus = "loading";
      http
        .patch(`/admin/communicationType/${id}`, {
          communicationType: communicationType.type,
          fields: communicationType.fields,
        })
        .then((result) => {
          this.updateStatus = { status: "success" };
          this.fetchCommunicationTypeById(id);
          this.fetchCommunicationTypes();
        })
        .catch((err) => {
          this.updateStatus = { status: "failed" };
        });
    },
    deleteCommunicationType(communicationType: number) {
      this.deleteStatus = "loading";
      http
        .delete(`/admin/communicationType/${communicationType}`)
        .then((res) => {
          this.deleteStatus = { status: "success" };
          this.fetchCommunicationTypes();
        })
        .catch((err) => {
          this.deleteStatus = { status: "failed", reason: err.data };
        });
    },
  },
});
