import { defineStore } from "pinia";
import type {
  CreateOrUpdateExecutivePositionViewModel,
  ExecutivePositionViewModel,
} from "../../viewmodels/admin/executivePosition.models";
import { http } from "../../serverCom";

export const useExecutivePositionStore = defineStore("executivePosition", {
  state: () => {
    return {
      executivePositions: [] as Array<ExecutivePositionViewModel>,
      executivePosition: null as null | ExecutivePositionViewModel,
      loadingAll: "loading" as "loading" | "fetched" | "failed",
      loadingSingle: "loading" as "loading" | "fetched" | "failed",
      createStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
      updateStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
      deleteStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
    };
  },
  actions: {
    resetStatus() {
      this.createStatus = null;
      this.updateStatus = null;
      this.deleteStatus = null;
    },
    fetchExecutivePositions() {
      this.loadingAll = "loading";
      http
        .get("/admin/executivePosition")
        .then((result) => {
          this.executivePositions = result.data;
          this.loadingAll = "fetched";
        })
        .catch((err) => {
          this.loadingAll = "failed";
        });
    },
    fetchExecutivePositionById(id: number) {
      this.executivePosition = null;
      this.loadingSingle = "loading";
      http
        .get(`/admin/executivePosition/${id}`)
        .then((result) => {
          this.executivePosition = result.data;
          this.loadingSingle = "fetched";
        })
        .catch((err) => {
          this.loadingSingle = "failed";
        });
    },
    createExecutivePosition(executivePosition: CreateOrUpdateExecutivePositionViewModel) {
      this.createStatus = "loading";
      http
        .post(`/admin/executivePosition`, {
          executivePosition: executivePosition.position,
        })
        .then((result) => {
          this.createStatus = { status: "success" };
          this.fetchExecutivePositions();
        })
        .catch((err) => {
          this.createStatus = { status: "failed" };
        });
    },
    updateActiveExecutivePosition(executivePosition: CreateOrUpdateExecutivePositionViewModel) {
      if (this.executivePosition == null) return;
      let id = this.executivePosition.id;
      this.updateStatus = "loading";
      http
        .patch(`/admin/executivePosition/${id}`, {
          executivePosition: executivePosition.position,
        })
        .then((result) => {
          this.updateStatus = { status: "success" };
          this.fetchExecutivePositionById(id);
          this.fetchExecutivePositions();
        })
        .catch((err) => {
          this.updateStatus = { status: "failed" };
        });
    },
    deleteExecutivePosition(executivePosition: number) {
      this.deleteStatus = "loading";
      http
        .delete(`/admin/executivePosition/${executivePosition}`)
        .then((res) => {
          this.deleteStatus = { status: "success" };
          this.fetchExecutivePositions();
        })
        .catch((err) => {
          this.deleteStatus = { status: "failed", reason: err.data };
        });
    },
  },
});
