import { defineStore } from "pinia";
import type {
  CreateOrUpdateQualificationViewModel,
  QualificationViewModel,
} from "../../viewmodels/admin/qualification.models";
import { http } from "../../serverCom";

export const useQualificationStore = defineStore("qualification", {
  state: () => {
    return {
      qualifications: [] as Array<QualificationViewModel>,
      qualification: null as null | QualificationViewModel,
      loadingAll: "loading" as "loading" | "fetched" | "failed",
      loadingSingle: "loading" as "loading" | "fetched" | "failed",
      createStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
      updateStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
      deleteStatus: null as null | "loading" | { status: "success" | "failed"; reason?: string },
    };
  },
  actions: {
    resetStatus() {
      this.createStatus = null;
      this.updateStatus = null;
      this.deleteStatus = null;
    },
    fetchQualifications() {
      this.loadingAll = "loading";
      http
        .get("/admin/qualification")
        .then((result) => {
          this.qualifications = result.data;
          this.loadingAll = "fetched";
        })
        .catch((err) => {
          this.loadingAll = "failed";
        });
    },
    fetchQualificationById(id: number) {
      this.qualification = null;
      this.loadingSingle = "loading";
      http
        .get(`/admin/qualification/${id}`)
        .then((result) => {
          this.qualification = result.data;
          this.loadingSingle = "fetched";
        })
        .catch((err) => {
          this.loadingSingle = "failed";
        });
    },
    createQualification(qualification: CreateOrUpdateQualificationViewModel) {
      this.createStatus = "loading";
      http
        .post(`/admin/qualification`, {
          qualification: qualification.qualification,
          description: qualification.description,
        })
        .then((result) => {
          this.createStatus = { status: "success" };
          this.fetchQualifications();
        })
        .catch((err) => {
          this.createStatus = { status: "failed" };
        });
    },
    updateActiveQualification(qualification: CreateOrUpdateQualificationViewModel) {
      if (this.qualification == null) return;
      let id = this.qualification.id;
      this.updateStatus = "loading";
      http
        .patch(`/admin/qualification/${id}`, {
          qualification: qualification.qualification,
          description: qualification.description,
        })
        .then((result) => {
          this.updateStatus = { status: "success" };
          this.fetchQualificationById(id);
          this.fetchQualifications();
        })
        .catch((err) => {
          this.updateStatus = { status: "failed" };
        });
    },
    deleteQualification(qualification: number) {
      this.deleteStatus = "loading";
      http
        .delete(`/admin/qualification/${qualification}`)
        .then((res) => {
          this.deleteStatus = { status: "success" };
          this.fetchQualifications();
        })
        .catch((err) => {
          this.deleteStatus = { status: "failed", reason: err.data };
        });
    },
  },
});
